import { splitProps } from "solid-js";
import { Dynamic } from "solid-js/web";
import { callEventHandler } from "./utils";
export function isTrivialHref(href) {
    return !href || href.trim() === "#";
}
export function useButtonProps({ tagName, disabled, href, target, rel, onClick, tabIndex = 0, type, }) {
    if (!tagName) {
        if (href != null || target != null || rel != null) {
            tagName = "a";
        }
        else {
            tagName = "button";
        }
    }
    const meta = { tagName };
    if (tagName === "button") {
        return [{ type: type || "button", disabled }, meta];
    }
    const handleClick = (event) => {
        if (disabled || (tagName === "a" && isTrivialHref(href))) {
            event.preventDefault();
        }
        if (disabled) {
            event.stopPropagation();
            return;
        }
        callEventHandler(onClick, event);
    };
    const handleKeyDown = (event) => {
        if (event.key === " ") {
            event.preventDefault();
            handleClick(event /*HACK calling click handler with keyboard event*/);
        }
    };
    return [
        {
            role: "button",
            // explicitly undefined so that it overrides the props disabled in a spread
            // e.g. <Tag {...props} {...hookProps} />
            disabled: undefined,
            tabIndex: disabled ? undefined : tabIndex,
            href: tagName === "a" && disabled ? undefined : href,
            target: tagName === "a" ? target : undefined,
            "aria-disabled": !disabled ? undefined : disabled,
            rel: tagName === "a" ? rel : undefined,
            onClick: handleClick,
            onKeyDown: handleKeyDown,
        },
        meta,
    ];
}
const Button = (props) => {
    const [local, otherProps] = splitProps(props, ["as"]);
    let a = props.tabIndex;
    const [buttonProps, { tagName }] = useButtonProps({
        tagName: local.as,
        ...otherProps,
    });
    return <Dynamic {...otherProps} {...buttonProps} as={tagName}/>;
};
Button.displayName = "Button";
export default Button;
